@@ -32,6 +32,8 @@
 #include "serial.h"
 #include "regcache.h"
 
+#include "m68k-tdep.h"
+
 static void dbug_open (char *args, int from_tty);
 
 static void
@@ -57,12 +59,12 @@ dbug_supply_register (char *regname, int regnamelen, char *val, int vallen)
     case 'D':
       if (regname[1] < '0' || regname[1] > '7')
 	return;
-      regno = regname[1] - '0' + D0_REGNUM;
+      regno = regname[1] - '0' + M68K_D0_REGNUM;
       break;
     case 'A':
       if (regname[1] < '0' || regname[1] > '7')
 	return;
-      regno = regname[1] - '0' + A0_REGNUM;
+      regno = regname[1] - '0' + M68K_A0_REGNUM;
       break;
     default:
       return;
@@ -158,6 +160,8 @@ dbug_open (char *args, int from_tty)
   monitor_open (args, &dbug_cmds, from_tty);
 }
 
+extern initialize_file_ftype _initialize_dbug_rom; /* -Wmissing-prototypes */
+
 void
 _initialize_dbug_rom (void)
 {
